@@ -19,7 +19,7 @@ developed by KaisaGlobal quant team.
 """
 
 
-import os, sys, json
+import os, sys, json, uuid
 
 import numpy as np
 import pandas as pd
@@ -31,8 +31,8 @@ import threading
 
 import websocket
 from copy import deepcopy
-import time
-
+import time, codecs
+from pub_cls_com import *
 
 from os.path import dirname, abspath
 
@@ -41,7 +41,7 @@ project_path = dirname(dirname(abspath(__file__)))
 projec_full_path = project_path + r'\common'
 sys.path.append(projec_full_path)
 
-
+import csv
 from com import *
 from KaisaCrypto import *
 from utils import *
@@ -50,6 +50,12 @@ from KaisaProtoc import *
 kcrypto = KaisaCrypto()
 kprotoc = KaisaProtoc()
 
+acc_id = 'sz1060'
+acc_id_path = os.getcwd().replace('\\', '/') + '/logger'
+_thisdate_ = datetime.datetime.now().strftime('%Y%m%d')
+##  ## 每十分钟运行一遍【5分钟后退订行情】//每个小时运行一遍【50分钟后退订行情】
+logger_path = r'/{}.csv'.format(datetime.datetime.now().strftime('%Y%m%d'))
+logger_filename = r'logger/_sheet_{}.csv'.format(_thisdate_)
 
 
 
@@ -81,11 +87,18 @@ class KaisaGateway(object):
 
     _market_connect_msg = ""
 
+    ## 所有字段--dic
+    ## 启动的时候记录接口初始化
+    ## 订阅的时候记录订阅
+    ## 行情推送的时候记录行情
+    ## 退订行情的时候记录退订
+    ## 在退订行情后直接将直接所有记录的数据写入到报表csv中
+    wiki_dict_ = {}
 
     def __init__(self):
         pass
 
-    def jq_user_config(self, user_id, user_pwd, account_code, account_pwd, openapi_token, ip_address=None, environment='paper'):
+    def jq_user_config(self, user_id, user_pwd, account_code, account_pwd, openapi_token, ip_address=None, environment='paper', callback=None):
 
         self.user_id = user_id
         self.user_pwd = user_pwd
@@ -98,8 +111,11 @@ class KaisaGateway(object):
 
         self.m_req_id = 0
         self.trade_req_id = 0
+        self.init_api_bool = False
         self.ip_address = ip_address
-
+        self.subscribe_code_list = []
+        self._market_ws = None
+        self.callback_cls = callback
 
     def _update_urls(self):
 
@@ -119,13 +135,14 @@ class KaisaGateway(object):
         self.TRADE_CRYPTO_RSA_URL = update_url(TRADE_CRYPTO_RSA_URL)
 
     def write_error(self, data):
-        print("error: {}".format(data))
+        print(f"error: {data}")
 
     def write_log(self, data):
-        print("log: {}".format(data))
+        print(f"log: {data}")
 
     def _authentica(self, auth_username: str, auth_password: str) -> None:
         """
+        ## websockt 登录
         获取网关登录令牌;
         :param auth_username:
         :param auth_password:
@@ -153,7 +170,10 @@ class KaisaGateway(object):
         if ipAddress:
             params["ipAddress"] = ipAddress
 
-        headerParam = {"srvType":"open","ipAddress":ipAddress,"phoneModel":"","deviceId":"","deviceToken":"","macAddress":"","channelId":"","country":"","sysVersion":"","apiVersion":"","platform":""}
+        headerParam = {"srvType": "open", "ipAddress": ipAddress, "phoneModel": "",
+                       "deviceId": "", "deviceToken": "", "macAddress": get_mac_addr(),
+                       "channelId": "", "country": "", "sysVersion": "",
+                       "apiVersion": "", "platform": ""}
         headerParam = json.dumps(headerParam)
 
         authorization_str = "basic {}".format(self.openapi_token)
@@ -164,6 +184,7 @@ class KaisaGateway(object):
             "Timestamp": timestamp_,
             "headerParam": headerParam
         }
+
         response = requests.post(
             url=self.AUTHENTICA_URL, params=params, headers=headers
         )
@@ -193,11 +214,72 @@ class KaisaGateway(object):
     def gateway_auth(self):
         return self._authentica(self.user_id, self.user_pwd)
 
+    def _mkfile(self, filename):
+        # 判断文件是否存在，不存在就创建
+        headers = ['update_time', 'id', 'is_sub', 'is_gateway_auth', 'is_init_api', 'is_un_sub', 'is_quote_arrive',
+                   'is_quote_right', 'tick_res_time', 'quote_res_time', 'broker_res_time', 'level_order_res_time',
+                   'is_api_error']
+        with open(filename, mode='w', encoding='UTF-8') as ff:
+            employee_writer = csv.writer(ff, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
+            employee_writer.writerow(headers)
+
+    def _mkdir(self, path):
+        # 判断文件夹是否存在，不存在就创建
+        if not os.path.exists(path):
+            os.mkdir(path)
+
+    def _make_file(self, filename):
+        # 建文件[如果文件不存在，建文件]
+        ff = open(filename, mode='w', encoding='UTF-8')
+        ff.close()
+
+    def _read_file(self, filename):
+        # 读取文件
+        df = pd.read_csv(filename)
+        return df
+
+    def _write_file(self, filename, data):
+        ## 追加写入文件
+        fs = open(filename, mode='a', encoding='utf-8')
+        fs.write(data)
+        fs.close()
+
+    def init_md_api(self):
+        '''
+        初始化mdApi
+        --login并验证行情账号
+        --初始化行情Api
+        '''
+
+        try:
+            self.wiki_dict_['id'] = 'sz1057'
+            ## 判断文件夹是否存在，不存在就创建
+            #self._mkdir(acc_id_path)
+            ## 判断文件是否存在，不存在就创建
+            self._make_file(logger_filename)
+            ret_ok, ret_msg = self.gateway_auth()
+            if ret_ok:
+                print('行情账号登录成功!')
+                self.init_api_bool = ret_ok
+                self.wiki_dict_['is_gateway_auth'] = 'True'
+                self.wiki_dict_['is_api_error'] = 'False'
+            else:
+                self.wiki_dict_['is_gateway_auth'] = 'False'
+                print('行情账号登录失败,error:{}!'.format(ret_msg))
+                ## -- 把行情推送中触发的异常写入log文档中[正常/失败]
+        except Exception as exp:
+            ## -- 把行情推送中触发的异常写入log文档中[正常/失败]
+            self.wiki_dict_['is_gateway_auth'] = 'False'
+            return 'market-ws disconnected!', exp
+
     def market_connect_ws(self):
-        ret_ok, ret_msg = self.gateway_auth()
-        if ret_ok:
-            self.market_connect_ws0()
-        time.sleep(5)
+        # connect ws
+        try:
+            if self.init_api_bool:
+                self.market_connect_ws0()
+
+        except Exception as exp:
+            return 'market-ws disconnected!', exp
 
     def do_market_requests(self, url, reqdata):
 
@@ -218,7 +300,6 @@ class KaisaGateway(object):
         resp_data = response.json()
         return status_code, resp_data
 
-
     def do_trade_requests(self, url, reqdata, reqtype="normal"):
 
         reqdata = self._trade_req_decorate(reqdata, reqtype)
@@ -238,7 +319,6 @@ class KaisaGateway(object):
         resp_data = response.json()
         return status_code, resp_data
 
-
     def _market_req_decorate(self, reqdata):
 
         reqdata['headers'] = {
@@ -267,7 +347,6 @@ class KaisaGateway(object):
 
         return contractdata_all
 
-
     def query_symbollist(self, beginpos=0, count=1000):
 
         self.req_id += 1
@@ -286,13 +365,12 @@ class KaisaGateway(object):
 
         reqdata = {"data": data}
         status_code, resp_data = self.do_market_requests(url=self.QUOTE_URL, reqdata=reqdata)
-        if status_code//100==2:
+        if status_code//100 == 2:
             symbols = resp_data['data']['symbol']
         else:
             symbols = None
         return symbols
 
-
     def query_history_kline(self, symbol='01638', market='HK', klinetype='1d', adjfactortype='0', pagenum=1, pagesize=100):
         """
         query history kline (only for hk market)
@@ -328,11 +406,32 @@ class KaisaGateway(object):
         if resp_status:
             body_data = resp_data['body']['records']
         else:
-            body_data = {'retCode': resp_data['retCode'],
-                         'retMsg': resp_data['retMsg']
-                         }
+            body_data = {
+                'retCode': resp_data['retCode'],
+                'retMsg': resp_data['retMsg']
+                }
         return resp_status, body_data
 
+    def get_history_kline_data(self, symbol=['00700'], start_date=None, end_date=None, unit='1d', market=1, fre='pre'):
+       '''
+       Args:
+            symbol: 合约列表 list
+            market: 市场类型：1 CH/ 2 HK/ 3 USA
+            unit: 频率 默认为 '1d' 将支持如下周期: '1m', '5m', '15m', '30m', '1d', '1w', '1M'. '1w' 表示一周, '1M' 表示一月
+            start_date: 开始日期
+            end_date: 结束日期
+            fre:是否复权  1 不复权/2 前复权/3 后复权
+        Returns: 返回历史日K/分钟K数据
+       '''
+        # /stock/quote/tickHistory/v2
+        # quoteInfo/kLineHistory/v2
+       post_url = HK_STOCK_KLINE_POST_URL
+       if unit != '1m' or unit != '1d':return '您好, 该接口暂时只提供日K和1分钟K线数据, 其他频率的数据, 接口维护团队会尽快补上!'
+       if unit == '1m':
+           if len(symbol) >1:return '您好, 检测到您填入多个合约获取历史分钟k线, 请修改后重新请求数据!'
+       if start_date is None and end_date is None:return '您好, 检测到您填入两个为None的日期, 请修改后重新请求数据!'
+       if market != 1:return '您好, 该接口暂时只提供港股数据, 请修改后重新请求数据!'
+       kprotoc.get_kline_hist_data_proto()
 
     def query_history_tick(self, symbol='00700', market='HK', pagesize=10, pagenum=1):
 
@@ -346,7 +445,7 @@ class KaisaGateway(object):
         response = requests.request(
             'POST',
             post_url,
-            headers={'Content-Type':'application/json'},
+            headers={'Content-Type': 'application/json'},
             params=None,
             data=json.dumps(rqdata)
         )
@@ -361,14 +460,12 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
-
-    def query_detailstate(self, symbol='01638', market='HK'):
-
-        post_url = MARKET_POST_URL+ "/quoteInfo/todayDetailState"
+    def query_detailstate(self, symbol='01638', market=2):
+        # detailstate
+        post_url = MARKET_POST_URL + "/quoteInfo/todayDetailState"
         rqdata = {
             'code': symbol,
-            'market': market,
+            'market': Quote_MARKET_DICT[market],
             'hasQuoteInfo': True,
             'hasQuoteExt': True,
             'hasQuoteStat': True,
@@ -378,7 +475,7 @@ class KaisaGateway(object):
         response = requests.request(
             'POST',
             post_url,
-            headers={'Content-Type': 'application/json', 'headerParam':json.dumps({'apiVersion':"1.2.0"})},
+            headers={'Content-Type': 'application/json', 'headerParam': json.dumps({'apiVersion': "1.2.0"})},
             params=None,
             data=json.dumps(rqdata)
         )
@@ -394,67 +491,203 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def market_connect_ws0(self):
-
         threading.Thread(target=self.__market_connect_ws, daemon=True).start()
-        #self.__market_connect_ws()
+        time.sleep(3)
 
     def get_market_connect_ws_status(self):
 
         return self._market_connect_status, self._market_connect_msg
 
+    def _on_message(self, ws, message) -> None:
+        # ws消息回报
+        #("### on_message ###")
+        self.req_id += 1
+        if message is None or message == '':
+            return
+        try:
+            self.wiki_dict_['is_quote_arrive'] = 'True'
+            data = kprotoc.rec_bstring_to_json(message)   # 把字符串转换为json数据类型
+            if data is None:
+                pass
+            elif data['data'] is not None:
+                self.callback_cls(data)
+
+                ##################################
+                self.wiki_dict_['is_quote_right'] = 'True'
+                if data['type'] == 'TICK':
+                    if 'tick_res_time' not in self.wiki_dict_:
+                        self.wiki_dict_['tick_res_time'] = [datetime.datetime.now()]
+                    else:
+                        tick_time_list = list(self.wiki_dict_['tick_res_time'])
+                        tick_time_list.append(datetime.datetime.now())     ## time.time()
+                        self.wiki_dict_['tick_res_time'] = tick_time_list
+
+                if data['type'] == 'QUOTE':
+                    if 'quote_res_time' not in self.wiki_dict_:
+                        self.wiki_dict_['quote_res_time'] = [datetime.datetime.now()]
+                    else:
+                        # print('quote_res_time:{}'.format(self.wiki_dict_['quote_res_time']))
+                        # print('quote_res_time--now:{}'.format(time.time()))
+                        quote_time_list = list(self.wiki_dict_['quote_res_time'])
+                        quote_time_list.append(datetime.datetime.now())
+                        self.wiki_dict_['quote_res_time'] = quote_time_list
+
+                if data['type'] == 'BROKER':
+                    if 'broker_res_time' not in self.wiki_dict_:
+                        self.wiki_dict_['broker_res_time'] = [datetime.datetime.now()]
+                    else:
+                        broker_time_list = list(self.wiki_dict_['broker_res_time'])
+                        broker_time_list.append(datetime.datetime.now())
+                        self.wiki_dict_['broker_res_time'] = broker_time_list
+                if data['type'] == 'ORDER':
+                    if 'level_order_res_time' not in self.wiki_dict_:
+                        self.wiki_dict_['level_order_res_time'] = [datetime.datetime.now()]
+                    else:
+                        order_time_list = list(self.wiki_dict_['level_order_res_time'])
+                        order_time_list.append(datetime.datetime.now())
+                        self.wiki_dict_['level_order_res_time'] = order_time_list
+                ##################################
+
+        except Exception as exp:
+            self.wiki_dict_['is_quote_right'] = 'False'
+            print('on-message-cls error:{}'.format(exp))
+            ## -- 把行情推送中触发的异常写入log文档中[正常/失败]
+            pass
+
+    def _on_error(self, ws, error) -> None:
+        # ws异常回报--把接口触发的异常写入log文档中[正常/失败]
+        self.wiki_dict_['is_api_error'] = 'True'
+        print(f'ws异常回报，error:{error}')
+
+    def _on_close(self, ws, close_status_code, close_msg) -> None:
+        # ws关闭连接(closed)回报
+        self.wiki_dict_['is_api_abnormal_close'] = 'True'
+        print(f"ws已关闭>> close_msg:{close_msg}!")
+
+    def _on_open(self, ws) -> None:
+        # ws open-sub
+        print("####### on_sub #######")
+        def run(*args):
+            time.sleep(1)
+            self.req_id += 1
+            sub_bstring = kprotoc.get_subscribe_bstring(self.subscribe_code_list, self.req_id)
+            self._market_ws.sock.send_binary(sub_bstring)
+            time.sleep(1)
+        try:
+            run()
+            th = threading.Thread(target=self._market_ping, args=())
+            th.start()
+            self.wiki_dict_['is_sub'] = 'True'
+        except Exception as exp:
+            ## -- 把每次订阅的行为结果写入log文档中[正常/失败]
+            print('on-open--error:{}'.format(exp))
+            self.wiki_dict_['is_sub'] = 'False'
+            pass
+
     def __market_connect_ws(self):
 
         if self._market_connect_status:
             return
-
         # header = {"Authorization": self.gatewayToken,
         #           "deviceId":"xxx",
         #           "platform":"open",
         #           "srvType":"open"}
 
         data = {"deviceId": "xxx",
-                  "platform": "open",
-                  "srvType": "open"}
+                "platform": "open",
+                "srvType": "open"}
 
         header = {"Authorization": self.gatewayToken,
-                  "headerParam": json.dumps(data)
-                    }
+                  "headerParam": json.dumps(data)}
 
         try:
-            self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
-            self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
-            self._market_ping_th.start()
+            ######################new++++++++++++++++++
+            #websocket.enableTrace(True)
+            self._market_ws = websocket.WebSocketApp(
+                url=MARKET_WS_URL,
+                header=header,
+                on_open=self._on_open,
+                on_message=self._on_message,
+                on_error=self._on_error,
+                on_close=self._on_close)
 
-            self._market_connect_status = True
+            #self._market_ws.on_open = self._on_open   # # ws open-ping
             self._market_connect_msg = ""
 
-            print('wait to receive.')
-            while True:
+            try:
+                self.wiki_dict_['is_init_api'] = 'True'
+                self._market_ws.run_forever()
+            except Exception as exp:
+                ## --把每次连接行情接口的操作结果写入log文档中[正常/失败]
+                print('__market_connect_ws:exp:{}'.format(exp))
+                self.wiki_dict_['is_init_api'] = 'False'
+                pass
+            ######################new++++++++++++++++++
 
-                text = self._market_ws.recv()
-                data = kprotoc.rec_bstring_to_json(text)
-                if data is None:
-                    continue
-                self.on_market_data(data)
+
+            ################旧版################
+            # self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
+            # print('ws连接状态:{}'.format(self._market_ws.status))
+            # print('ws连接connected:{}'.format(self._market_ws.connected))
+            # # self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
+            # # self._market_ping_th.start()
+            # self._market_connect_status = self._market_ws.connected
+            # self._market_connect_msg = ""
+            # time.sleep(5)
+            # ret_status, ret_data = self.get_market_connect_ws_status()
+            # if ret_status:
+            #     print("market-ws connected!")
+            # else:
+            #     print('market-ws disconnected, error:{}'.format(ret_data))
+            # while True:
+            #     print('_market_ws--> recv >> ')
+            #     text = self._market_ws.recv()              # 接口实时返回的字符串
+            #     print('__market_connect_ws >> text:{}'.format(text))
+            #     if text is None or text == '':
+            #         continue
+            #     data = kprotoc.rec_bstring_to_json(text)   # 把字符串转换为json数据类型
+            #     if data is None:
+            #         continue
+            #
+            #     self.on_market_data(data)
+            #################旧版################
         except Exception as exp:
-            print('__market_connect_ws:exp:{}'.format(exp))
+            ## --把每次连接行情接口的操作结果写入log文档中[正常/失败]
+            print('WebSocket Connection Status:{}'.format(self._market_ws.sock))
+            print('market-ws disconnected, error:{}'.format(exp))
+            self.wiki_dict_['is_init_api'] = 'False'
 
     def on_market_packet(self, packet):
-
         self.on_market_data(packet)
 
     def on_market_data(self, data):
+        # 行情数据到达>>data
         pass
 
     def subscribe_marketdata(self, code_list):
         """
-        推送行情和分笔；
+        订阅quote和tick数据
+        code_list: [{'code':'01638', 'market':2, 'sub_type':2}]
         """
+
+        sub_types = [data['sub_type'] for data in code_list]
+        sub_types = list(set(sub_types))
+        if QUOTE in sub_types:
+            self.wiki_dict_['quote_res_time'] = [datetime.datetime.now()]  # time.time()
+
+        if TICK in sub_types:
+            self.wiki_dict_['tick_res_time'] = [datetime.datetime.now()]
+
+        if LEVEL_ORDER in sub_types:
+            self.wiki_dict_['level_order_res_time'] = [datetime.datetime.now()]
+
+        if BROKERS in sub_types:
+            self.wiki_dict_['broker_res_time'] = [datetime.datetime.now()]
+
+        self.subscribe_code_list = code_list
         self.req_id += 1
-        bstring = kprotoc.get_subscribe_bstring(code_list, self.req_id)
-        self._market_ws.send_binary(bstring)
+        self.market_connect_ws0()
 
     def unsubscribe_marketdata(self, code_list):
         """
@@ -462,15 +695,35 @@ class KaisaGateway(object):
         :param symbols:
         :return:
         """
-        self.req_id+=1
-        bstring = kprotoc.get_unsubscribe_bstring(code_list, self.req_id)
-        self._market_ws.send_binary(bstring)
+        try:
+            self.req_id += 1
+            bstring = kprotoc.get_unsubscribe_bstring(code_list, self.req_id)
+            self._market_ws.sock.send_binary(bstring)
+            self.wiki_dict_['is_un_sub'] = 'True'
+        except Exception as exp:
+            ## --把每次退订行情的操作结果写入log文档中[正常/失败]
+            self.wiki_dict_['is_un_sub'] = 'False'
+            pass
+        self.wiki_dict_['update_time'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
+
+    def _market_ping(self):
+        """
+        :param this_ws:
+        :return:
+        """
+        while True:
+            self.market_heartbeat()
+            time.sleep(5)
 
     def market_heartbeat(self):
 
         self.req_id += 1
+
         ping_bstring = kprotoc.get_ping_bstring(self.req_id)
-        self._market_ws.send_binary(ping_bstring)
+
+        self._market_ws.sock.send_binary(ping_bstring)
+
+        #self._market_ws.send_binary(ping_bstring)
 
     def send_market_packet(self, packet):
 
@@ -488,7 +741,6 @@ class KaisaGateway(object):
         }
         return req
 
-
     def _trade_req_decorate(self, reqdata, reqtype="normal"):
 
         json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
@@ -499,40 +751,40 @@ class KaisaGateway(object):
 
         this_timestamp = generate_timestamp()
 
-        if (reqtype=="connect") and (method=="POST") and ('q' in reqdata['data']):
+        if (reqtype == "connect") and (method == "POST") and ('q' in reqdata['data']):
             headers = {"Content-Type": "application/json",
                        "Authorization": self.token
-                        }
+                       }
             params = deepcopy(reqdata['data'])
-            reqdata = {'method':method,
-                       'headers':headers,
-                       'data':json_dumps(reqdata['data']),
+            reqdata = {'method': method,
+                       'headers': headers,
+                       'data': json_dumps(reqdata['data']),
                        'params': params
                        }
             return reqdata
 
-        if (reqtype=="login"):
+        if reqtype == "login":
 
             headers['Content-Type'] = "application/json"
             headers["Authorization"] = self.token
-            if reqtype=="login":
+            if reqtype == "login":
                 data['Authorization'] = self.token
             if self.tradeToken is not None:
                 headers['X-Trade-Token'] = self.tradeToken
-            if reqtype=="normal":
+            if reqtype == "normal":
                 if 'tradeToken' in data:
                     del data['tradeToken']
-            if method=="POST":
+            if method == "POST":
                 header_ = reqdata['data'].copy()
                 if self.tradeToken is not None:
                     header_["tradeToken"] = self.tradeToken
                 headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
                 headers["Timestamp"] = this_timestamp
 
-            reqdata = {"method":method,
-                       "data":json_dumps(data),
-                       "headers":headers,
-                       "params":json_dumps(params)}
+            reqdata = {"method": method,
+                       "data": json_dumps(data),
+                       "headers": headers,
+                       "params": json_dumps(params)}
             return reqdata
 
         headers = {"Content-Type": "application/json",
@@ -540,7 +792,7 @@ class KaisaGateway(object):
                    "X-Trade-Token": self.tradeToken
                    }
 
-        if method=="GET":
+        if method == "GET":
             if 'params' in reqdata:
                 header_ = reqdata['params'].copy()
             else:
@@ -549,22 +801,22 @@ class KaisaGateway(object):
             headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="GET")
             headers["Timestamp"] = this_timestamp
 
-        if method=="POST":
+        if method == "POST":
             header_ = data.copy()
             if self.tradeToken is not None:
                 header_["tradeToken"] = self.tradeToken
             headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
             headers["Timestamp"] = this_timestamp
 
-        if (method=="POST") and ('q' not in data):
+        if (method == "POST") and ('q' not in data):
             q_string = json_dumps(data)
             q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
             data = {'q': q_string}
 
-        reqdata = {"method":method,
-                   "data":json_dumps(data),
-                   "headers":headers,
-                   "params":params}
+        reqdata = {"method": method,
+                   "data": json_dumps(data),
+                   "headers": headers,
+                   "params": params}
 
         return reqdata
 
@@ -576,33 +828,33 @@ class KaisaGateway(object):
         data = reqdata['data'] if 'data' in reqdata else {}
         params = reqdata['params'] if 'params' in reqdata else {}
 
-        if (reqtype=="connect") and (method=="POST") and ('q' in reqdata['data']):
+        if (reqtype == "connect") and (method == "POST") and ('q' in reqdata['data']):
             headers = {"Content-Type": "application/json",
-                       "Authorization": self.token
-                        }
+                       "Authorization": self.token}
+
             params = deepcopy(reqdata['data'])
-            reqdata = {'method':method,
-                       'headers':headers,
-                       'data':json_dumps(reqdata['data']),
+            reqdata = {'method': method,
+                       'headers': headers,
+                       'data': json_dumps(reqdata['data']),
                        'params': params
                        }
             return reqdata
 
-        if (reqtype=="login"):
+        if reqtype == "login":
 
             headers['Content-Type'] = "application/json"
             headers["Authorization"] = self.token
-            if reqtype=="login":
+            if reqtype == "login":
                 data['Authorization'] = self.token
 
             if self.tradeToken is not None:
                 headers['X-Trade-Token'] = self.tradeToken
 
-            if reqtype=="normal":
+            if reqtype == "normal":
                 if 'tradeToken' in data:
                     del data['tradeToken']
 
-            if method=="POST":
+            if method == "POST":
                 timestamp_ = generate_timestamp()
                 header_ = reqdata['data'].copy()
                 if self.tradeToken is not None:
@@ -610,10 +862,10 @@ class KaisaGateway(object):
                 headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
                 headers["Timestamp"] = timestamp_
 
-            reqdata = {"method":method,
-                       "data":json_dumps(data),
-                       "headers":headers,
-                       "params":json_dumps(params)}
+            reqdata = {"method": method,
+                       "data": json_dumps(data),
+                       "headers": headers,
+                       "params": json_dumps(params)}
 
             return reqdata
 
@@ -622,14 +874,14 @@ class KaisaGateway(object):
                    "X-Trade-Token": self.tradeToken
                    }
 
-        if method=="GET":
+        if method == "GET":
             timestamp_ = generate_timestamp()
             header_ = reqdata['params'].copy()
             header_["Authorization"] = self.token
             headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="GET")
             headers["Timestamp"] = timestamp_
 
-        if method=="POST":
+        if method == "POST":
             timestamp_ = generate_timestamp()
             header_ = data.copy()
             if self.tradeToken is not None:
@@ -637,19 +889,18 @@ class KaisaGateway(object):
             headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
             headers["Timestamp"] = timestamp_
 
-        if (method=="POST") and ('q' not in data):
+        if (method == "POST") and ('q' not in data):
             q_string = json_dumps(data)
             q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
             data = {'q': q_string}
 
-        reqdata = {"method":method,
-                   "data":json_dumps(data),
-                   "headers":headers,
-                   "params":params}
+        reqdata = {"method": method,
+                   "data": json_dumps(data),
+                   "headers": headers,
+                   "params": params}
 
         return reqdata
 
-
     def __sign(self, header_: dict = {}, request_=None, timestamp_=None):
 
         sign_ = None
@@ -660,7 +911,6 @@ class KaisaGateway(object):
         sign_ = kcrypto.encrypt_md5(str_)
         return sign_
 
-
     def trade_connect(self):
         """
         交易连接
@@ -684,7 +934,6 @@ class KaisaGateway(object):
         self.trade_connect_msg = ''
         return self.trade_connect_status, self.trade_connect_msg
 
-
     def trade_auth(self):
 
         data = {
@@ -708,22 +957,12 @@ class KaisaGateway(object):
         self.trade_auth_msg = ''
         return self.trade_auth_status, self.trade_auth_msg
 
-
     def trade_connect_ws(self):
         threading.Thread(target=self._start_trade_ws, daemon=True).start()
 
-
     def get_trade_connect_ws_status(self):
         return self._ws_trade_connect_status, ""
 
-    def _market_ping(self, this_ws):
-        """
-        :param this_ws:
-        :return:
-        """
-        while True:
-            self.market_heartbeat()
-            time.sleep(5)
 
     def _trade_ping(self, this_ws):
         """
@@ -732,7 +971,6 @@ class KaisaGateway(object):
         """
         pass
 
-
     def _start_trade_ws(self):
 
         if self._ws_trade_connect_status:
@@ -765,7 +1003,6 @@ class KaisaGateway(object):
             text = self._trade_ws.recv()
             self.on_trade_packet(text)
 
-
     def on_trade_packet(self, packet):
 
         if len(packet)==0:
@@ -779,7 +1016,6 @@ class KaisaGateway(object):
             data = json.loads(packet)
             self.on_trade_data(packet)
 
-
     def send_order(self, bsFlag='B', price=550, qty=600, code='00700'):
 
         return self.place_order(bsFlag=bsFlag, price=price, qty=qty, code=code)
@@ -809,7 +1045,6 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def cancel_order(self, sys_orderid):
 
         data = {
@@ -833,7 +1068,6 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def query_portfolio(self):
 
         reqdata = {"method":"GET",
@@ -851,7 +1085,6 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def query_balance(self):
 
         reqdata = {"method": "GET",
@@ -868,7 +1101,6 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def query_position(self):
 
         reqdata = {"method": "GET",
@@ -885,7 +1117,6 @@ class KaisaGateway(object):
                          }
         return resp_status, body_data
 
-
     def query_order(self):
 
         reqdata = {"method": "GET",
